"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable @typescript-eslint/no-non-null-assertion */
const inspector_1 = __importDefault(require("inspector"));
const pg_1 = require("pg");
const inbox_service_1 = require("./inbox-service");
const logger_1 = require("./logger");
const inboxSpy = __importStar(require("./inbox"));
const isDebugMode = () => inspector_1.default.url() !== undefined;
if (isDebugMode()) {
    jest.setTimeout(600000);
}
else {
    (0, logger_1.disableLogger)(); // Hide logs if the tests are not run in debug mode
    jest.setTimeout(7000);
}
const ackInboxSpy = jest.spyOn(inboxSpy, 'ackInbox');
const nackInboxSpy = jest.spyOn(inboxSpy, 'nackInbox');
const repService = {};
jest.mock('pg-logical-replication', () => {
    return Object.assign(Object.assign({}, jest.requireActual('pg-logical-replication')), { LogicalReplicationService: jest.fn().mockImplementation(() => {
            const lrs = {
                // on: jest.fn(),
                handleData: undefined,
                handleError: undefined,
                on: (event, listener) => {
                    switch (event) {
                        case 'data':
                            repService.handleData = listener;
                            break;
                        case 'error':
                            repService.handleError = listener;
                            break;
                    }
                },
                acknowledge: jest.fn(),
                removeAllListeners: jest.fn(),
                emit: jest.fn(),
                stop: jest.fn(() => Promise.resolve()),
                subscribe: () => new Promise(() => {
                    /** never return */
                }),
                isStop: () => true,
            };
            repService.acknowledge = lrs.acknowledge;
            repService.stop = lrs.stop;
            return lrs;
        }) });
});
jest.mock('pg', () => {
    return Object.assign(Object.assign({}, jest.requireActual('pg')), { Pool: jest.fn().mockImplementation(() => ({
            connect: jest.fn(() => new pg_1.Client()),
            on: jest.fn(),
            end: jest.fn(() => Promise.resolve()),
            removeAllListeners: jest.fn(),
        })), Client: jest.fn().mockImplementation(() => ({
            query: jest.fn((sql, params) => __awaiter(void 0, void 0, void 0, function* () {
                switch (sql) {
                    case 'SELECT processed_at FROM test_schema.test_table WHERE id = $1 FOR UPDATE NOWAIT': {
                        const dbMessage = inboxDbMessage(params[0]);
                        return {
                            rowCount: dbMessage ? 1 : 0,
                            rows: dbMessage ? [{ processed_at: dbMessage.processed_at }] : [],
                        };
                    }
                }
            })),
            on: jest.fn(),
            release: jest.fn(),
        })) });
});
jest.mock('./utils', () => {
    return Object.assign(Object.assign({}, jest.requireActual('./utils')), { executeTransaction: jest.fn((pool, callback) => __awaiter(void 0, void 0, void 0, function* () {
            const client = yield pool.connect();
            const response = yield callback(client);
            client.release();
            return response;
        })) });
});
const aggregate_type = 'test_type';
const event_type = 'test_event_type';
const inboxDbMessage = (id) => [
    {
        id: 'not_processed_id',
        aggregate_type,
        event_type,
        aggregate_id: 'test_aggregate_id',
        payload: { result: 'success' },
        created_at: new Date('2023-01-18T21:02:27.000Z'),
        retries: 2,
        processed_at: null,
    },
    {
        id: 'processed_id',
        aggregate_type,
        event_type,
        aggregate_id: 'test_aggregate_id',
        payload: { result: 'success' },
        created_at: new Date('2023-01-18T21:02:27.000Z'),
        retries: 0,
        processed_at: new Date('2023-01-18T21:02:27.000Z'),
    },
    {
        id: 'retries_exceeded',
        aggregate_type,
        event_type,
        aggregate_id: 'test_aggregate_id',
        payload: { result: 'success' },
        created_at: new Date('2023-01-18T21:02:27.000Z'),
        retries: 4,
        processed_at: null,
    },
].find((m) => m.id === id);
const config = {
    pgConfig: {
        host: 'test_host',
        port: 5432,
        database: 'test_db',
        user: 'test_user',
        password: 'test_password',
    },
    pgReplicationConfig: {
        host: 'test_host',
        port: 5432,
        database: 'test_db',
        user: 'test_inbox_user',
        password: 'test_inbox_user_password',
    },
    settings: {
        dbSchema: 'test_schema',
        dbTable: 'test_table',
        postgresPub: 'test_pub',
        postgresSlot: 'test_slot',
    },
};
const relation = {
    tag: 'relation',
    relationOid: 1,
    schema: 'test_schema',
    name: 'test_table',
    replicaIdentity: 'default',
    columns: [
        {
            name: 'id',
            flags: 0,
            typeOid: 23,
            typeMod: -1,
            typeSchema: 'pg_catalog',
            typeName: 'int4',
            parser: (raw) => raw,
        },
    ],
    keyColumns: ['id'],
};
describe('Inbox service unit tests - initializeInboxService', () => {
    beforeEach(() => {
        repService.handleData = undefined;
        repService.handleError = undefined;
        repService.acknowledge = undefined;
        repService.stop = undefined;
        ackInboxSpy.mockReset();
        nackInboxSpy.mockReset();
    });
    it('should call the correct messageHandler and acknowledge the WAL message when no errors are thrown', () => __awaiter(void 0, void 0, void 0, function* () {
        // Arrange
        const messageHandler = jest.fn(() => Promise.resolve());
        const unusedMessageHandler = jest.fn(() => Promise.resolve());
        const message = {
            id: 'not_processed_id',
            aggregateType: aggregate_type,
            aggregateId: 'test_aggregate_id',
            eventType: event_type,
            payload: { result: 'success' },
            createdAt: '2023-01-18T21:02:27.000Z',
            retries: 2,
        };
        const [cleanup] = yield (0, inbox_service_1.initializeInboxService)(config, [
            {
                aggregateType: message.aggregateType,
                eventType: message.eventType,
                handle: messageHandler,
            },
            {
                aggregateType: 'unused-aggregate-type',
                eventType: 'unused-event',
                handle: unusedMessageHandler,
            },
        ]);
        expect(repService.handleData).toBeDefined();
        expect(repService.handleError).toBeDefined();
        // Act
        yield repService.handleData('0/00000001', {
            tag: 'insert',
            relation,
            new: inboxDbMessage('not_processed_id'),
        });
        // Assert
        expect(messageHandler).toHaveBeenCalledWith(message, expect.any(Object));
        expect(unusedMessageHandler).not.toHaveBeenCalled();
        expect(repService.acknowledge).toHaveBeenCalledWith('0/00000001');
        expect(ackInboxSpy).toHaveBeenCalledWith(message, expect.any(Object), expect.any(Object));
        expect(nackInboxSpy).not.toHaveBeenCalled();
        expect(repService.stop).not.toHaveBeenCalled();
        yield cleanup();
        expect(repService.stop).toHaveBeenCalledTimes(1);
    }));
    it('should call all the correct messageHandlers and acknowledge the WAL message when no errors are thrown', () => __awaiter(void 0, void 0, void 0, function* () {
        // Arrange
        const messageHandler1 = jest.fn(() => Promise.resolve());
        const messageHandler2 = jest.fn(() => Promise.resolve());
        const unusedMessageHandler = jest.fn(() => Promise.resolve());
        const message = {
            id: 'not_processed_id',
            aggregateType: aggregate_type,
            aggregateId: 'test_aggregate_id',
            eventType: event_type,
            payload: { result: 'success' },
            createdAt: '2023-01-18T21:02:27.000Z',
            retries: 2,
        };
        const [cleanup] = yield (0, inbox_service_1.initializeInboxService)(config, [
            {
                aggregateType: message.aggregateType,
                eventType: message.eventType,
                handle: messageHandler1,
            },
            {
                aggregateType: message.aggregateType,
                eventType: message.eventType,
                handle: messageHandler2,
            },
            {
                aggregateType: 'unused-aggregate-type',
                eventType: 'unused-event',
                handle: unusedMessageHandler,
            },
        ]);
        expect(repService.handleData).toBeDefined();
        expect(repService.handleError).toBeDefined();
        // Act
        yield repService.handleData('0/00000001', {
            tag: 'insert',
            relation,
            new: inboxDbMessage('not_processed_id'),
        });
        // Assert
        expect(messageHandler1).toHaveBeenCalledWith(message, expect.any(Object));
        expect(messageHandler2).toHaveBeenCalledWith(message, expect.any(Object));
        expect(ackInboxSpy).toHaveBeenCalledWith(message, expect.any(Object), expect.any(Object));
        expect(nackInboxSpy).not.toHaveBeenCalled();
        expect(unusedMessageHandler).not.toHaveBeenCalled();
        expect(repService.acknowledge).toHaveBeenCalledWith('0/00000001');
        expect(repService.stop).not.toHaveBeenCalled();
        yield cleanup();
        expect(repService.stop).toHaveBeenCalledTimes(1);
    }));
    it.each(['processed_id', 'not_found'])('should not call a messageHandler but acknowledge the WAL message when the inbox DB message was already process or not found: %p', (messageId) => __awaiter(void 0, void 0, void 0, function* () {
        // Arrange
        const messageHandler = jest.fn(() => Promise.resolve());
        const unusedMessageHandler = jest.fn(() => Promise.resolve());
        const message = {
            id: messageId,
            aggregateType: aggregate_type,
            aggregateId: 'test_aggregate_id',
            eventType: event_type,
            payload: { result: 'success' },
            createdAt: '2023-01-18T21:02:27.000Z',
            retries: 2,
        };
        const [cleanup] = yield (0, inbox_service_1.initializeInboxService)(config, [
            {
                aggregateType: message.aggregateType,
                eventType: message.eventType,
                handle: messageHandler,
            },
            {
                aggregateType: 'unused-aggregate-type',
                eventType: 'unused-event',
                handle: unusedMessageHandler,
            },
        ]);
        expect(repService.handleData).toBeDefined();
        expect(repService.handleError).toBeDefined();
        // Act
        yield repService.handleData('0/00000001', {
            tag: 'insert',
            relation,
            new: inboxDbMessage('processed_id'),
        });
        // Assert
        expect(messageHandler).not.toHaveBeenCalled();
        expect(unusedMessageHandler).not.toHaveBeenCalled();
        expect(repService.acknowledge).toHaveBeenCalledWith('0/00000001');
        // As the inbox DB message was not found or already processed --> no ack/nack needed
        expect(ackInboxSpy).not.toHaveBeenCalled();
        expect(nackInboxSpy).not.toHaveBeenCalled();
        expect(repService.stop).not.toHaveBeenCalled();
        yield cleanup();
        expect(repService.stop).toHaveBeenCalledTimes(1);
    }));
    it('should call a messageHandler on a not processed message but not acknowledge the WAL message when the message handler throws an error', () => __awaiter(void 0, void 0, void 0, function* () {
        // Arrange
        const unusedMessageHandler = jest.fn(() => Promise.resolve());
        const message = {
            id: 'not_processed_id',
            aggregateType: aggregate_type,
            aggregateId: 'test_aggregate_id',
            eventType: event_type,
            payload: { result: 'success' },
            createdAt: '2023-01-18T21:02:27.000Z',
            retries: 2,
        };
        const [cleanup] = yield (0, inbox_service_1.initializeInboxService)(config, [
            {
                aggregateType: message.aggregateType,
                eventType: message.eventType,
                handle: () => __awaiter(void 0, void 0, void 0, function* () {
                    throw new Error('Unit Test');
                }),
            },
            {
                aggregateType: 'unused-aggregate-type',
                eventType: 'unused-event',
                handle: unusedMessageHandler,
            },
        ]);
        expect(repService.handleData).toBeDefined();
        expect(repService.handleError).toBeDefined();
        // Act
        yield repService.handleData('0/00000001', {
            tag: 'insert',
            relation,
            new: inboxDbMessage('not_processed_id'),
        });
        // Assert
        expect(unusedMessageHandler).not.toHaveBeenCalled();
        expect(repService.acknowledge).not.toHaveBeenCalled();
        expect(ackInboxSpy).not.toHaveBeenCalled();
        expect(nackInboxSpy).toHaveBeenCalledWith(message, expect.any(Object), expect.any(Object));
        expect(repService.stop).not.toHaveBeenCalled();
        yield cleanup();
        expect(repService.stop).toHaveBeenCalledTimes(1);
    }));
    it('should call a messageHandler on a message that has reached the retry limit and thus acknowledge the WAL message when the message handler throws an error', () => __awaiter(void 0, void 0, void 0, function* () {
        // Arrange
        const unusedMessageHandler = jest.fn(() => Promise.resolve());
        const message = {
            id: 'retries_exceeded',
            aggregateType: aggregate_type,
            eventType: event_type,
            aggregateId: 'test_aggregate_id',
            payload: { result: 'success' },
            createdAt: '2023-01-18T21:02:27.000Z',
            retries: 4,
        };
        const [cleanup] = yield (0, inbox_service_1.initializeInboxService)(config, [
            {
                aggregateType: message.aggregateType,
                eventType: message.eventType,
                handle: () => __awaiter(void 0, void 0, void 0, function* () {
                    throw new Error('Unit Test');
                }),
            },
            {
                aggregateType: 'unused-aggregate-type',
                eventType: 'unused-event',
                handle: unusedMessageHandler,
            },
        ]);
        expect(repService.handleData).toBeDefined();
        expect(repService.handleError).toBeDefined();
        // Act
        yield repService.handleData('0/00000001', {
            tag: 'insert',
            relation,
            new: inboxDbMessage('retries_exceeded'),
        });
        // Assert
        expect(unusedMessageHandler).not.toHaveBeenCalled();
        expect(repService.acknowledge).not.toHaveBeenCalled();
        expect(ackInboxSpy).not.toHaveBeenCalled();
        expect(nackInboxSpy).toHaveBeenCalledWith(message, expect.any(Object), expect.any(Object));
        expect(repService.stop).not.toHaveBeenCalled();
        yield cleanup();
        expect(repService.stop).toHaveBeenCalledTimes(1);
    }));
});
//# sourceMappingURL=inbox-service.spec.js.map