"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable @typescript-eslint/no-non-null-assertion */
const inspector_1 = __importDefault(require("inspector"));
const local_replication_service_1 = require("./local-replication-service");
const logger_1 = require("./logger");
const utils_1 = require("./utils");
const isDebugMode = () => inspector_1.default.url() !== undefined;
if (isDebugMode()) {
    jest.setTimeout(600000);
}
else {
    jest.setTimeout(7000); // for the 5sec heartbeat
    (0, logger_1.disableLogger)(); // Hide logs if the tests are not run in debug mode
}
const repService = {};
jest.mock('pg-logical-replication', () => {
    return Object.assign(Object.assign({}, jest.requireActual('pg-logical-replication')), { LogicalReplicationService: jest.fn().mockImplementation(() => {
            const lrs = {
                handleData: undefined,
                handleError: undefined,
                handleHeartbeat: undefined,
                on: (event, listener) => {
                    switch (event) {
                        case 'data':
                            repService.handleData = listener;
                            break;
                        case 'error':
                            repService.handleError = listener;
                            break;
                        case 'heartbeat':
                            repService.handleHeartbeat = listener;
                            break;
                    }
                },
                acknowledge: jest.fn(),
                removeAllListeners: jest.fn(),
                emit: jest.fn(),
                stop: jest.fn(() => Promise.resolve()),
                subscribe: () => new Promise(() => {
                    /** never return */
                }),
                isStop: () => true,
            };
            repService.acknowledge = lrs.acknowledge;
            repService.stop = lrs.stop;
            return lrs;
        }) });
});
const settings = {
    dbSchema: 'test_schema',
    dbTable: 'test_table',
    postgresPub: 'test_pub',
    postgresSlot: 'test_slot',
};
const relation = {
    tag: 'relation',
    relationOid: 1,
    schema: 'test_schema',
    name: 'test_table',
    replicaIdentity: 'default',
    columns: [
        {
            name: 'id',
            flags: 0,
            typeOid: 23,
            typeMod: -1,
            typeSchema: 'pg_catalog',
            typeName: 'int4',
            parser: (raw) => raw,
        },
        {
            name: 'name',
            flags: 0,
            typeOid: 25,
            typeMod: -1,
            typeSchema: 'pg_catalog',
            typeName: 'text',
            parser: (raw) => raw,
        },
    ],
    keyColumns: ['id'],
};
describe('Local replication service unit tests', () => {
    describe('getRelevantMessage', () => {
        const mapAdditionalRows = jest.fn();
        it('should return undefined if log tag is not "insert"', () => {
            var _a;
            // Arrange
            const update = {
                tag: 'update',
                relation: relation,
                key: { id: 1 },
                old: { id: 1, name: 'old_name' },
                new: { id: 1, name: 'new_name' },
            };
            // Act
            const message = (_a = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.getRelevantMessage) === null || _a === void 0 ? void 0 : _a.call(local_replication_service_1.__only_for_unit_tests__, update, settings);
            // Assert
            expect(message).toBeUndefined();
        });
        it('should return undefined if log relation schema is not the same as settings', () => {
            var _a;
            // Arrange
            const log = {
                tag: 'insert',
                relation: Object.assign(Object.assign({}, relation), { schema: 'wrong_schema' }),
                new: {},
            };
            // Act
            const message = (_a = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.getRelevantMessage) === null || _a === void 0 ? void 0 : _a.call(local_replication_service_1.__only_for_unit_tests__, log, settings);
            // Assert
            expect(message).toBeUndefined();
        });
        it('should return undefined if log relation name is not the same as settings', () => {
            var _a;
            // Arrange
            const log = {
                tag: 'insert',
                relation: Object.assign(Object.assign({}, relation), { name: 'wrong_name' }),
                new: {},
            };
            // Act
            const message = (_a = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.getRelevantMessage) === null || _a === void 0 ? void 0 : _a.call(local_replication_service_1.__only_for_unit_tests__, log, settings);
            // Assert
            expect(message).toBeUndefined();
        });
        it('should return the outbox message if log relation schema and name are the same as settings', () => {
            var _a;
            // Arrange
            const log = {
                tag: 'insert',
                relation,
                new: {
                    id: 'test_id',
                    aggregate_type: 'test_type',
                    aggregate_id: 'test_aggregate_id',
                    event_type: 'test_event_type',
                    payload: { test: 'payload' },
                    created_at: new Date('2023-01-18T21:02:27.000Z'),
                },
            };
            // Act
            const message = (_a = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.getRelevantMessage) === null || _a === void 0 ? void 0 : _a.call(local_replication_service_1.__only_for_unit_tests__, log, settings, mapAdditionalRows);
            // Assert
            expect(message).toEqual({
                id: 'test_id',
                aggregateType: 'test_type',
                aggregateId: 'test_aggregate_id',
                eventType: 'test_event_type',
                payload: { test: 'payload' },
                createdAt: '2023-01-18T21:02:27.000Z',
            });
            expect(mapAdditionalRows).toHaveBeenCalledWith({
                id: 'test_id',
                aggregate_type: 'test_type',
                aggregate_id: 'test_aggregate_id',
                event_type: 'test_event_type',
                payload: { test: 'payload' },
                created_at: new Date('2023-01-18T21:02:27.000Z'),
            });
        });
    });
    describe('mapMessage', () => {
        it('should return undefined if input is not an object', () => {
            var _a, _b, _c;
            // Act + Assert
            expect((_a = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.mapMessage) === null || _a === void 0 ? void 0 : _a.call(local_replication_service_1.__only_for_unit_tests__, 'not an object')).toBeUndefined();
            expect((_b = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.mapMessage) === null || _b === void 0 ? void 0 : _b.call(local_replication_service_1.__only_for_unit_tests__, null)).toBeUndefined();
            expect((_c = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.mapMessage) === null || _c === void 0 ? void 0 : _c.call(local_replication_service_1.__only_for_unit_tests__, undefined)).toBeUndefined();
        });
        it('should return undefined if input is missing required properties', () => {
            var _a, _b;
            // Act + Assert
            expect((_a = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.mapMessage) === null || _a === void 0 ? void 0 : _a.call(local_replication_service_1.__only_for_unit_tests__, {})).toBeUndefined();
            expect((_b = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.mapMessage) === null || _b === void 0 ? void 0 : _b.call(local_replication_service_1.__only_for_unit_tests__, {
                id: '123',
                aggregate_type: 'test_type',
            })).toBeUndefined();
        });
        it('should return undefined if input has wrong types for required properties', () => {
            var _a;
            // Act + Assert
            expect((_a = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.mapMessage) === null || _a === void 0 ? void 0 : _a.call(local_replication_service_1.__only_for_unit_tests__, {
                id: 123,
                aggregate_type: 'test_type',
                aggregate_id: 'test_id',
                event_type: 'test_event',
                created_at: 'not a date',
                payload: { test: 'payload' },
            })).toBeUndefined();
        });
        it('should return an OutboxMessage if input is valid', () => {
            var _a;
            // Arrange
            const input = {
                id: '123',
                aggregate_type: 'test_type',
                aggregate_id: 'test_id',
                event_type: 'test_event',
                created_at: new Date('2023-01-18T21:02:27.000Z'),
                payload: { test: 'payload' },
            };
            // Act
            const message = (_a = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.mapMessage) === null || _a === void 0 ? void 0 : _a.call(local_replication_service_1.__only_for_unit_tests__, input);
            // Assert
            expect(message).toBeDefined();
            expect(message).toEqual({
                id: input.id,
                aggregateType: input.aggregate_type,
                aggregateId: input.aggregate_id,
                eventType: input.event_type,
                payload: input.payload,
                createdAt: '2023-01-18T21:02:27.000Z',
            });
        });
        it('should map additional properties if mapAdditionalRows is provided', () => {
            var _a;
            // Arrange
            const input = {
                id: '123',
                aggregate_type: 'test_type',
                aggregate_id: 'test_id',
                event_type: 'test_event',
                created_at: new Date('2023-01-18T21:02:27.000Z'),
                payload: { test: 'payload' },
                additional_property: 'additional_value',
            };
            const mapAdditionalRows = (row) => ({
                additional_property: row.additional_property,
            });
            // Act
            const message = (_a = local_replication_service_1.__only_for_unit_tests__ === null || local_replication_service_1.__only_for_unit_tests__ === void 0 ? void 0 : local_replication_service_1.__only_for_unit_tests__.mapMessage) === null || _a === void 0 ? void 0 : _a.call(local_replication_service_1.__only_for_unit_tests__, input, mapAdditionalRows);
            // Assert
            expect(message).toBeDefined();
            expect(message).toEqual({
                id: '123',
                aggregateType: 'test_type',
                aggregateId: 'test_id',
                eventType: 'test_event',
                payload: { test: 'payload' },
                createdAt: '2023-01-18T21:02:27.000Z',
                additional_property: 'additional_value',
            });
        });
    });
    describe('createService', () => {
        let messageHandler;
        let errorHandler;
        let mapAdditionalRows;
        beforeEach(() => {
            messageHandler = jest.fn();
            errorHandler = jest.fn();
            mapAdditionalRows = jest.fn();
            repService.handleData = undefined;
            repService.handleError = undefined;
            repService.handleHeartbeat = undefined;
            repService.acknowledge = undefined;
            repService.stop = undefined;
        });
        it('should call messageHandler and acknowledge the message when no errors are thrown', () => __awaiter(void 0, void 0, void 0, function* () {
            // Arrange
            const config = {
                pgReplicationConfig: {},
                settings,
            };
            const [cleanup] = yield (0, local_replication_service_1.createService)(config, messageHandler, errorHandler, mapAdditionalRows);
            const message = {
                id: 'test_id',
                aggregateType: 'test_type',
                aggregateId: 'test_aggregate_id',
                eventType: 'test_event_type',
                payload: { result: 'success' },
                createdAt: '2023-01-18T21:02:27.000Z',
            };
            expect(repService.handleData).toBeDefined();
            // Act
            yield repService.handleData('0/00000001', {
                tag: 'insert',
                relation,
                new: {
                    id: 'test_id',
                    aggregate_type: 'test_type',
                    aggregate_id: 'test_aggregate_id',
                    event_type: 'test_event_type',
                    payload: { result: 'success' },
                    created_at: new Date('2023-01-18T21:02:27.000Z'),
                },
            });
            // Assert
            expect(repService.handleError).toBeDefined();
            expect(repService.handleHeartbeat).toBeDefined();
            expect(messageHandler).toHaveBeenCalledWith(message);
            expect(errorHandler).not.toHaveBeenCalled();
            expect(mapAdditionalRows).toHaveBeenCalled();
            expect(repService.acknowledge).toHaveBeenCalledWith('0/00000001');
            expect(repService.stop).not.toHaveBeenCalled();
            yield cleanup();
            expect(repService.stop).toHaveBeenCalledTimes(1);
        }));
        it('should call messageHandler but not acknowledge the message when an error is thrown', () => __awaiter(void 0, void 0, void 0, function* () {
            // Arrange
            const config = {
                pgReplicationConfig: {},
                settings,
            };
            const testError = new Error('Transient error');
            let errorHandlerCalled = false;
            const [cleanup] = yield (0, local_replication_service_1.createService)(config, () => __awaiter(void 0, void 0, void 0, function* () {
                throw testError;
            }), (msg, error) => __awaiter(void 0, void 0, void 0, function* () {
                expect(msg).toStrictEqual(message);
                expect(error).toStrictEqual(testError);
                errorHandlerCalled = true;
                return true;
            }), mapAdditionalRows);
            const message = {
                id: 'test_id',
                aggregateType: 'test_type',
                aggregateId: 'test_aggregate_id',
                eventType: 'test_event_type',
                payload: { result: 'success' },
                createdAt: '2023-01-18T21:02:27.000Z',
            };
            expect(repService.handleData).toBeDefined();
            // Act
            yield repService.handleData('0/00000001', {
                tag: 'insert',
                relation,
                new: {
                    id: 'test_id',
                    aggregate_type: 'test_type',
                    aggregate_id: 'test_aggregate_id',
                    event_type: 'test_event_type',
                    payload: { result: 'success' },
                    created_at: new Date('2023-01-18T21:02:27.000Z'),
                },
            });
            // Assert
            expect(repService.handleError).toBeDefined();
            expect(repService.handleHeartbeat).toBeDefined();
            expect(errorHandlerCalled).toBe(true);
            expect(mapAdditionalRows).toHaveBeenCalled();
            expect(errorHandler).not.toHaveBeenCalled();
            expect(repService.acknowledge).not.toHaveBeenCalled();
            expect(repService.stop).not.toHaveBeenCalled();
            yield cleanup();
            expect(repService.stop).toHaveBeenCalledTimes(1);
        }));
        it('should call messageHandler and not acknowledge the message when an error is thrown', () => __awaiter(void 0, void 0, void 0, function* () {
            // Arrange
            const config = {
                pgReplicationConfig: {},
                settings,
            };
            const testError = new Error('Unit test error');
            let errorHandlerCalled = false;
            const [cleanup] = yield (0, local_replication_service_1.createService)(config, () => __awaiter(void 0, void 0, void 0, function* () {
                throw testError;
            }), (msg, error) => __awaiter(void 0, void 0, void 0, function* () {
                expect(msg).toStrictEqual(message);
                expect(error).toStrictEqual(testError);
                errorHandlerCalled = true;
                return true;
            }), mapAdditionalRows);
            const message = {
                id: 'test_id',
                aggregateType: 'test_type',
                aggregateId: 'test_aggregate_id',
                eventType: 'test_event_type',
                payload: { result: 'success' },
                createdAt: '2023-01-18T21:02:27.000Z',
            };
            expect(repService.handleData).toBeDefined();
            // Act
            yield repService.handleData('0/00000001', {
                tag: 'insert',
                relation,
                new: {
                    id: 'test_id',
                    aggregate_type: 'test_type',
                    aggregate_id: 'test_aggregate_id',
                    event_type: 'test_event_type',
                    payload: { result: 'success' },
                    created_at: new Date('2023-01-18T21:02:27.000Z'),
                },
            });
            // Assert
            expect(repService.handleError).toBeDefined();
            expect(repService.handleHeartbeat).toBeDefined();
            expect(errorHandlerCalled).toBe(true);
            expect(errorHandler).not.toHaveBeenCalled();
            expect(mapAdditionalRows).toHaveBeenCalled();
            expect(repService.acknowledge).not.toHaveBeenCalled();
            expect(repService.stop).not.toHaveBeenCalled();
            yield cleanup();
            expect(repService.stop).toHaveBeenCalledTimes(1);
        }));
        it('A heartbeat should be acknowledged after 5 seconds', () => __awaiter(void 0, void 0, void 0, function* () {
            // Arrange
            const config = {
                pgReplicationConfig: {},
                settings,
            };
            const [cleanup] = yield (0, local_replication_service_1.createService)(config, messageHandler, errorHandler, mapAdditionalRows);
            expect(repService.handleHeartbeat).toBeDefined();
            // Act
            yield repService.handleHeartbeat('0/00000001', 123, true);
            // Assert
            expect(repService.handleData).toBeDefined();
            expect(repService.handleError).toBeDefined();
            expect(messageHandler).not.toHaveBeenCalled();
            expect(mapAdditionalRows).not.toHaveBeenCalled();
            expect(errorHandler).not.toHaveBeenCalled();
            expect(repService.acknowledge).not.toHaveBeenCalled();
            yield (0, utils_1.sleep)(4000);
            expect(repService.acknowledge).not.toHaveBeenCalled();
            yield (0, utils_1.sleep)(1010);
            expect(repService.acknowledge).toHaveBeenCalled();
            expect(repService.stop).not.toHaveBeenCalled();
            yield cleanup();
            expect(repService.stop).toHaveBeenCalledTimes(1);
        }));
        it('A heartbeat should not be acknowledged after 5 seconds when a message acknowledgement comes in between', () => __awaiter(void 0, void 0, void 0, function* () {
            // Arrange
            const config = {
                pgReplicationConfig: {},
                settings,
            };
            const [cleanup] = yield (0, local_replication_service_1.createService)(config, messageHandler, errorHandler, mapAdditionalRows);
            expect(repService.handleData).toBeDefined();
            expect(repService.handleHeartbeat).toBeDefined();
            // Act
            yield repService.handleHeartbeat('0/00000001', 123, true);
            yield (0, utils_1.sleep)(1000);
            yield repService.handleData('0/00000002', {
                tag: 'insert',
                relation,
                new: {
                    id: 'test_id',
                    aggregate_type: 'test_type',
                    aggregate_id: 'test_aggregate_id',
                    event_type: 'test_event_type',
                    payload: { result: 'success' },
                    created_at: new Date('2023-01-18T21:02:27.000Z'),
                },
            });
            // Assert
            expect(repService.handleError).toBeDefined();
            expect(messageHandler).toHaveBeenCalled();
            expect(mapAdditionalRows).toHaveBeenCalled();
            expect(errorHandler).not.toHaveBeenCalled();
            expect(repService.acknowledge).toHaveBeenCalledWith('0/00000002');
            yield (0, utils_1.sleep)(4010);
            expect(repService.acknowledge).toHaveBeenCalledTimes(1);
            expect(repService.stop).not.toHaveBeenCalled();
            yield cleanup();
            expect(repService.stop).toHaveBeenCalledTimes(1);
        }));
    });
});
//# sourceMappingURL=local-replication-service.spec.js.map