"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-non-null-assertion */
const inspector_1 = __importDefault(require("inspector"));
const logger_1 = require("./logger");
const pg_1 = require("pg");
const outbox_1 = require("./outbox");
const isDebugMode = () => inspector_1.default.url() !== undefined;
if (isDebugMode()) {
    jest.setTimeout(600000);
}
else {
    (0, logger_1.disableLogger)(); // Hide logs if the tests are not run in debug mode
    jest.setTimeout(7000);
}
const config = {
    pgReplicationConfig: {
        host: 'test_host',
        port: 5432,
        database: 'test_db',
        user: 'test_outbox_user',
        password: 'test_outbox_user_password',
    },
    settings: {
        dbSchema: 'test_schema',
        dbTable: 'test_table',
        postgresPub: 'test_pub',
        postgresSlot: 'test_slot',
    },
};
// required for the initializeOutboxMessageStorage
jest.mock('pg', () => {
    return Object.assign(Object.assign({}, jest.requireActual('pg')), { Pool: jest.fn().mockImplementation(() => ({
            connect: jest.fn(() => new pg_1.Client()),
            on: jest.fn(),
            end: jest.fn(() => Promise.resolve()),
            removeAllListeners: jest.fn(),
        })), Client: jest.fn().mockImplementation(() => ({
            query: jest.fn((sql, _params) => __awaiter(void 0, void 0, void 0, function* () {
                if (sql.indexOf(`INSERT INTO ${config.settings.dbSchema}.${config.settings.dbTable}`)) {
                    return { rowCount: 1 };
                }
                throw new Error(`Missed to mock the following SQL query: ${sql}`);
            })),
            on: jest.fn(),
            release: jest.fn(),
        })) });
});
describe('Outbox unit tests', () => {
    describe('initializeOutboxMessageStorage', () => {
        it('should store the outbox message data to the database', () => __awaiter(void 0, void 0, void 0, function* () {
            // Arrange
            const pool = new pg_1.Pool();
            const client = yield pool.connect();
            client.query = jest
                .fn()
                .mockReturnValueOnce({
                rowCount: 1,
                rows: [{ created_at: new Date() }],
            })
                .mockReturnValueOnce({});
            const aggregateId = 'test-aggregate-id';
            const aggregateType = 'movie';
            const eventType = 'created';
            const payload = { test: 'data' };
            const storeOutboxMessage = (0, outbox_1.initializeOutboxMessageStorage)(aggregateType, eventType, config);
            // Act
            const result = yield storeOutboxMessage(aggregateId, payload, client);
            // Assert
            expect(client.query).toHaveBeenCalledTimes(2);
            expect(client.query).toHaveBeenCalledWith(expect.stringContaining(`INSERT INTO ${config.settings.dbSchema}.${config.settings.dbTable}`), [expect.any(String), aggregateType, aggregateId, eventType, payload]);
            expect(result).toEqual({
                id: expect.any(String),
                aggregateType,
                aggregateId,
                eventType,
                payload,
                createdAt: expect.any(Date),
            });
        }));
        it('should throw an error if the outbox message could not be created', () => __awaiter(void 0, void 0, void 0, function* () {
            // Arrange
            const pool = new pg_1.Pool();
            const client = yield pool.connect();
            client.query = jest.fn().mockReturnValue({ rowCount: 0 });
            const storeOutboxMessage = (0, outbox_1.initializeOutboxMessageStorage)('movie', 'created', config);
            // Act + Assert
            yield expect(storeOutboxMessage('test-aggregate-id', { test: 'data' }, client)).rejects.toThrow('Could not insert the message into the outbox!');
        }));
    });
});
//# sourceMappingURL=outbox.spec.js.map