import { Pool, PoolClient } from 'pg';
/**
 * Returns the error as verified Error object or wraps the input as Error.
 * @param error The error variable to check
 * @returns The error if the input was already an error otherwise a wrapped error.
 */
export declare const ensureError: (error: unknown) => Error;
/**
 * Sleep for a given amount of milliseconds
 * @param milliseconds The time in milliseconds to sleep
 * @returns The (void) promise to await
 */
export declare const sleep: (milliseconds: number) => Promise<void>;
/**
 * Open a transaction and execute the callback as part of the transaction.
 * @param pool The PostgreSQL database pool
 * @param callback The callback to execute DB commands with.
 * @returns The result of the callback (if any).
 * @throws Any error from the database or the callback.
 */
export declare const executeTransaction: <T>(pool: Pool, callback: (client: PoolClient) => Promise<T>) => Promise<T>;
//# sourceMappingURL=utils.d.ts.map